# -*- coding: utf-8 -*-

#  PycaWM
#  basedirspec.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  Code based on basedir.py from ROX-Lib2
#  Copyright (C) 2002  Thomas Leonard (http://rox.sourceforge.net)
#  Relicensed under GPL 3, as allowed by part 3 of the LGPL 2.1

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""The freedesktop.org Base Directory specification provides a way for
applications to locate shared data and configuration:

    http://standards.freedesktop.org/basedir-spec/

This module can be used to load and save from and to these directories.

Typical usage:

    from pycawm import basedirspec

    for dir in basedirspec.load_config_paths('MyOptions', 'Option'):
        print "Load settings from", dir

    dir = basedirspec.save_config_path('MyOptions')
    print >>file(os.path.join(dir, 'Option'), 'w'), "foo=2"
"""

import sys

if sys.version_info < (2, 5):
    raise ImportError('basedirspec module needs at least Python 2.5!')

from os import getenv, makedirs
from os.path import exists, isdir, join as path_join


# default xdg directories

_home = getenv('HOME', default='/')
xdg_data_home = getenv('XDG_DATA_HOME',
                       default=path_join(_home, '.local', 'share'))

xdg_data_dirs = [xdg_data_home] + getenv('XDG_DATA_DIRS',
                                         default=('/usr/local/share:'
                                                  '/usr/share')).split(':')

xdg_config_home = getenv('XDG_CONFIG_HOME',
                         default=path_join(_home, '.config'))

xdg_config_dirs = [xdg_config_home] + getenv('XDG_CONFIG_DIRS',
                                             default='/etc/xdg').split(':')

xdg_data_dirs = filter(lambda x: x, xdg_data_dirs)
xdg_config_dirs = filter(lambda x: x, xdg_config_dirs)

# PycaWM specific directories

_subdir = 'PycaWM'
pycawm_config_home = path_join(xdg_config_home, _subdir)
pycawm_data_home = path_join(xdg_data_home, _subdir)
pycawm_data_dirs = map(lambda x: path_join(x, _subdir), xdg_data_dirs)
pycawm_config_dirs = map(lambda x: path_join(x, _subdir), xdg_config_dirs)

# all these functions are PycaWM specific

def save_config_path(*resource):
    """Ensure $XDG_CONFIG_HOME/PycaWM/<resource>/ exists, and return its path.
    'resource' must be the name of a directory inside the PycaWM config path.
    Use this when SAVING configuration settings. Use the pycawm_config_dirs
    variable for loading."""
    path = pycawm_config_home
    if resource:
        resource = path_join(*resource)
        assert not resource.startswith('/')
        path = path_join(path, resource)
    if not isdir(path):
        makedirs(path, 0700)
    return path

def save_data_path(*resource):
    """Ensure $XDG_DATA_HOME/PycaWM/<resource>/ exists, and return its path.
    'resource' must be the name of a directory inside the PycaWM data path.
    Use this when updating a shared (between programs) database. Use the
    pycawm_data_dirs variable for loading."""
    path = pycawm_data_home
    if resource:
        resource = path_join(*resource)
        assert not resource.startswith('/')
        path = path_join(path, resource)
    if not isdir(path):
        makedirs(path)
    return path

def load_config_paths(*resource):
    """Returns an iterator which gives each directory named 'resource' in the
    configuration search path. Information provided by earlier directories
    should take precedence over later ones (ie, the user's config dir comes
    first)."""
    resource = path_join(*resource)
    if resource.startswith('/'):
        if exists(resource):
            yield resource
    else:
        for config_dir in pycawm_config_dirs:
            path = path_join(config_dir, resource)
            if exists(path):
                yield path

def load_first_config(*resource):
    """Returns the first result from load_config_paths,
    or None if there is nothing to load."""
    for x in load_config_paths(*resource):
        return x
    return None

def load_data_paths(*resource):
    """Returns an iterator which gives each directory named 'resource' in the
    shared data search path. Information provided by earlier directories should
    take precedence over later ones."""
    resource = path_join(*resource)
    if resource.startswith('/'):
        if exists(resource):
            yield resource
    else:
        for data_dir in pycawm_data_dirs:
            path = path_join(data_dir, resource)
            if exists(path):
                yield path

def load_first_data(*resource):
    """Returns the first result from load_data_paths,
    or None if there is nothing to load."""
    for x in load_data_paths(*resource):
        return x
    return None
